package sun.print;

import javax.print.attribute.PrintRequestAttributeSet;
import java.awt.Window;
import java.awt.print.PrinterJob;
import sun.awt.windows.WPrinterJob;
import javax.print.ServiceUIFactory;
import javax.print.attribute.HashAttributeSet;
import javax.print.attribute.standard.CopiesSupported;
import javax.print.attribute.AttributeSet;
import javax.print.attribute.standard.Fidelity;
import javax.print.attribute.standard.SheetCollate;
import javax.print.attribute.standard.RequestingUserName;
import java.net.URISyntaxException;
import java.net.URI;
import java.io.File;
import javax.print.attribute.standard.Destination;
import javax.print.attribute.standard.Media;
import javax.print.attribute.standard.PageRanges;
import javax.print.attribute.standard.OrientationRequested;
import javax.print.attribute.standard.JobName;
import javax.print.attribute.standard.Chromaticity;
import javax.print.attribute.standard.PrintQuality;
import javax.print.attribute.standard.Sides;
import javax.print.attribute.standard.ColorSupported;
import javax.print.attribute.PrintServiceAttribute;
import javax.print.event.PrintServiceAttributeListener;
import javax.print.attribute.Attribute;
import javax.print.attribute.AttributeSetUtilities;
import javax.print.attribute.HashPrintServiceAttributeSet;
import javax.print.DocPrintJob;
import javax.print.attribute.standard.Copies;
import javax.print.attribute.standard.QueuedJobCount;
import javax.print.attribute.standard.Severity;
import javax.print.attribute.standard.PrinterStateReason;
import javax.print.attribute.standard.PrinterStateReasons;
import javax.print.attribute.standard.PrinterState;
import javax.print.attribute.standard.PrinterIsAcceptingJobs;
import java.util.Iterator;
import java.util.ArrayList;
import java.util.HashMap;
import javax.print.attribute.standard.PrinterResolution;
import javax.print.attribute.standard.MediaPrintableArea;
import javax.print.attribute.PrintServiceAttributeSet;
import javax.print.attribute.standard.PrinterName;
import javax.print.attribute.standard.MediaTray;
import javax.print.attribute.standard.MediaSizeName;
import javax.print.DocFlavor;
import javax.print.attribute.standard.MediaSize;
import javax.print.PrintService;

public class Win32PrintService implements PrintService, AttributeUpdater, SunPrinterJobService
{
    public static MediaSize[] predefMedia;
    private static final DocFlavor[] supportedFlavors;
    private static final Class[] serviceAttrCats;
    private static Class[] otherAttrCats;
    public static final MediaSizeName[] dmPaperToPrintService;
    private static final MediaTray[] dmPaperBinToPrintService;
    private static int DM_PAPERSIZE;
    private static int DM_PRINTQUALITY;
    private static int DM_YRESOLUTION;
    private static final int DMRES_MEDIUM = -3;
    private static final int DMRES_HIGH = -4;
    private static final int DMORIENT_LANDSCAPE = 2;
    private static final int DMDUP_VERTICAL = 2;
    private static final int DMDUP_HORIZONTAL = 3;
    private static final int DMCOLLATE_TRUE = 1;
    private static final int DMCOLOR_MONOCHROME = 1;
    private static final int DMCOLOR_COLOR = 2;
    private static final int DMPAPER_A2 = 66;
    private static final int DMPAPER_A6 = 70;
    private static final int DMPAPER_B6_JIS = 88;
    private static final int DEVCAP_COLOR = 1;
    private static final int DEVCAP_DUPLEX = 2;
    private static final int DEVCAP_COLLATE = 4;
    private static final int DEVCAP_QUALITY = 8;
    private static final int DEVCAP_POSTSCRIPT = 16;
    private String printer;
    private PrinterName name;
    private String port;
    private transient PrintServiceAttributeSet lastSet;
    private transient ServiceNotifier notifier;
    private MediaSizeName[] mediaSizeNames;
    private MediaPrintableArea[] mediaPrintables;
    private MediaTray[] mediaTrays;
    private PrinterResolution[] printRes;
    private HashMap mpaMap;
    private int nCopies;
    private int prnCaps;
    private int[] defaultSettings;
    private boolean gotTrays;
    private boolean gotCopies;
    private boolean mediaInitialized;
    private boolean mpaListInitialized;
    private ArrayList idList;
    private MediaSize[] mediaSizes;
    private boolean isInvalid;
    private Win32PrintService.Win32DocumentPropertiesUI docPropertiesUI;
    private Win32PrintService.Win32ServiceUIFactory uiFactory;
    
    Win32PrintService(String printer) {
        this.notifier = null;
        this.docPropertiesUI = null;
        this.uiFactory = null;
        if (printer == null) {
            throw new IllegalArgumentException("null printer name");
        }
        this.printer = printer;
        this.mediaInitialized = false;
        this.gotTrays = false;
        this.gotCopies = false;
        this.isInvalid = false;
        this.printRes = null;
        this.prnCaps = 0;
        this.defaultSettings = null;
        this.port = null;
        return;
    }
    
    public void invalidateService() {
        this.isInvalid = true;
    }
    
    @Override
    public String getName() {
        return this.printer;
    }
    
    private PrinterName getPrinterName() {
        if (this.name == null) {
            this.name = new PrinterName(this.printer, null);
        }
        return this.name;
    }
    
    public int findPaperID(MediaSizeName object) {
        if ((object instanceof Win32MediaSize)) {
            return ((Win32MediaSize)object).getDMPaper();
        }
        int i = 0;
        while (i < Win32PrintService.dmPaperToPrintService.length) {
            if (Win32PrintService.dmPaperToPrintService[i].equals(object)) {
                return i + 1;
            }
            ++i;
        }
        if (object.equals(MediaSizeName.ISO_A2)) {
            return 66;
        }
        if (object.equals(MediaSizeName.ISO_A6)) {
            return 70;
        }
        if (object.equals(MediaSizeName.JIS_B6)) {
            return 88;
        }
        this.initMedia();
        if (this.idList != null && this.mediaSizes != null && this.idList.size() == this.mediaSizes.length) {
            int j = 0;
            while (j < this.idList.size()) {
                if (this.mediaSizes[j].getMediaSizeName() == object) {
                    return (int)this.idList.get(j);
                }
                ++j;
            }
        }
        return 0;
    }
    
    public int findTrayID(MediaTray mediaTray) {
        this.getMediaTrays();
        if ((mediaTray instanceof Win32MediaTray)) {
            return ((Win32MediaTray)mediaTray).getDMBinID();
        }
        int i = 0;
        while (i < Win32PrintService.dmPaperBinToPrintService.length) {
            if (mediaTray.equals(Win32PrintService.dmPaperBinToPrintService[i])) {
                return i + 1;
            }
            ++i;
        }
        return 0;
    }
    
    public MediaTray findMediaTray(int n) {
        if (n >= 1 && n <= Win32PrintService.dmPaperBinToPrintService.length) {
            return Win32PrintService.dmPaperBinToPrintService[n - 1];
        }
        MediaTray[] mediaTrays = this.getMediaTrays();
        if (mediaTrays != null) {
            for (int i = 0; i < mediaTrays.length; ++i) {
                if ((mediaTrays[i] instanceof Win32MediaTray)) {
                    Win32MediaTray win32MediaTray = (Win32MediaTray)mediaTrays[i];
                    if (win32MediaTray.winID == n) {
                        return win32MediaTray;
                    }
                }
            }
        }
        return Win32MediaTray.AUTO;
    }
    
    public MediaSizeName findWin32Media(int n) {
        if (n >= 1 && n <= Win32PrintService.dmPaperToPrintService.length) {
            return Win32PrintService.dmPaperToPrintService[n - 1];
        }
        switch (n) {
            case 66: {
                return MediaSizeName.ISO_A2;
            }
            case 70: {
                return MediaSizeName.ISO_A6;
            }
            case 88: {
                return MediaSizeName.JIS_B6;
            }
            default: {
                return null;
            }
        }
    }
    
    private boolean addToUniqueList(ArrayList list, MediaSizeName e) {
        int i = 0;
        while (i < list.size()) {
            if ((MediaSizeName)list.get(i) == e) {
                return false;
            }
            ++i;
        }
        list.add(e);
        return true;
    }
    
    private synchronized void initMedia() {
        if (this.mediaInitialized) {
            return;
        }
        this.mediaInitialized = true;
        int[] allMediaIDs = this.getAllMediaIDs(this.printer, this.getPort());
        if (allMediaIDs == null) {
            return;
        }
        ArrayList<Object> list = new ArrayList();
        ArrayList<Win32MediaSize> list2 = new ArrayList();
        ArrayList<Object> list3 = new ArrayList();
        this.idList = new ArrayList();
        for (int i = 0; i < allMediaIDs.length; ++i) {
            this.idList.add(Integer.valueOf(allMediaIDs[i]));
        }
        ArrayList<String> list4 = new ArrayList();
        this.mediaSizes = this.getMediaSizes(this.idList, allMediaIDs, list4);
        for (int j = 0; j < this.idList.size(); ++j) {
            MediaSizeName media = this.findWin32Media((int)this.idList.get(j));
            if (media != null && this.idList.size() == this.mediaSizes.length) {
                MediaSize mediaSizeForName = MediaSize.getMediaSizeForName(media);
                MediaSize mediaSize = this.mediaSizes[j];
                int n = 2540;
                if (Math.abs(mediaSizeForName.getX(1) - mediaSize.getX(1)) > (float)n || Math.abs(mediaSizeForName.getY(1) - mediaSize.getY(1)) > (float)n) {
                    media = null;
                }
            }
            int n2 = media != null;
            if (media == null && this.idList.size() == this.mediaSizes.length) {
                media = this.mediaSizes[j].getMediaSizeName();
            }
            boolean addToUniqueList = false;
            if (media != null) {
                addToUniqueList = this.addToUniqueList(list, media);
            }
            if ((n2 == 0 || addToUniqueList == 0) && this.idList.size() == list4.size()) {
                Win32MediaSize mediaName = Win32MediaSize.findMediaName((String)list4.get(j));
                if (mediaName == null && this.idList.size() == this.mediaSizes.length) {
                    mediaName = new Win32MediaSize((String)list4.get(j), (int)this.idList.get(j));
                    this.mediaSizes[j] = new MediaSize(this.mediaSizes[j].getX(1000), this.mediaSizes[j].getY(1000), 1000, mediaName);
                }
                if (mediaName != null && mediaName != media) {
                    if (addToUniqueList == 0) {
                        this.addToUniqueList(list, mediaName);
                    }
                    else {
                        list2.add(mediaName);
                    }
                }
            }
        }
        Iterator<Object> iterator = list2.iterator();
        while (iterator.hasNext()) {
            this.addToUniqueList(list, (Win32MediaSize)iterator.next());
        }
        list.toArray(this.mediaSizeNames = new MediaSizeName[list.size()]);
        return;
    }
    
    private synchronized MediaPrintableArea[] getMediaPrintables(MediaSizeName mediaSizeName) {
        if (mediaSizeName == null) {
            if (this.mpaListInitialized) {
                return this.mediaPrintables;
            }
        }
        else if (this.mpaMap != null && this.mpaMap.get(mediaSizeName) != null) {
            return new MediaPrintableArea[] { (MediaPrintableArea)this.mpaMap.get(mediaSizeName) };
        }
        this.initMedia();
        if (this.mediaSizeNames == null || this.mediaSizeNames.length == 0) {
            return null;
        }
        MediaSizeName[] mediaSizeNames;
        if (mediaSizeName != null) {
            mediaSizeNames = new MediaSizeName[] { mediaSizeName };
        }
        else {
            mediaSizeNames = this.mediaSizeNames;
        }
        if (this.mpaMap == null) {
            this.mpaMap = new HashMap();
        }
        for (int i = 0; i < mediaSizeNames.length; ++i) {
            MediaSizeName mediaSizeName2 = mediaSizeNames[i];
            if (this.mpaMap.get(mediaSizeName2) == null && mediaSizeName2 != null) {
                int paperID = this.findPaperID(mediaSizeName2);
                float[] array = (paperID != 0) ? this.getMediaPrintableArea(this.printer, paperID) : null;
                if (array != null) {
                    try {
                        this.mpaMap.put(mediaSizeName2, new MediaPrintableArea(array[0], array[1], array[2], array[3], 25400));
                    }
                    catch (IllegalArgumentException ex2) {}
                }
                else {
                    MediaSize mediaSizeForName = MediaSize.getMediaSizeForName(mediaSizeName2);
                    if (mediaSizeForName != null) {
                        try {
                            this.mpaMap.put(mediaSizeName2, new MediaPrintableArea(0.0f, 0.0f, mediaSizeForName.getX(25400), mediaSizeForName.getY(25400), 25400));
                        }
                        catch (IllegalArgumentException ex) {}
                    }
                }
            }
        }
        if (this.mpaMap.size() == 0) {
            return null;
        }
        if (mediaSizeName != null) {
            if (this.mpaMap.get(mediaSizeName) == null) {
                return null;
            }
            return new MediaPrintableArea[] { (MediaPrintableArea)this.mpaMap.get(mediaSizeName) };
        }
        else {
            this.mediaPrintables = (MediaPrintableArea[])(MediaPrintableArea[])this.mpaMap.values().toArray(new MediaPrintableArea[0]);
            this.mpaListInitialized = true;
            return this.mediaPrintables;
        }
    }
    
    private synchronized MediaTray[] getMediaTrays() {
        if (this.gotTrays && this.mediaTrays != null) {
            return this.mediaTrays;
        }
        String port = this.getPort();
        int[] allMediaTrays = this.getAllMediaTrays(this.printer, port);
        String[] allMediaTrayNames = this.getAllMediaTrayNames(this.printer, port);
        if (allMediaTrays == null || allMediaTrayNames == null) {
            return null;
        }
        int n = 0;
        for (int i = 0; i < allMediaTrays.length; ++i) {
            if (allMediaTrays[i] > 0) {
                ++n;
            }
        }
        MediaTray[] mediaTrays = new MediaTray[n];
        int j = 0;
        int n2 = 0;
        while (j < Math.min(allMediaTrays.length, allMediaTrayNames.length)) {
            int n3 = allMediaTrays[j];
            if (n3 > 0) {
                if (n3 > Win32PrintService.dmPaperBinToPrintService.length || Win32PrintService.dmPaperBinToPrintService[n3 - 1] == null) {
                    mediaTrays[n2++] = new Win32MediaTray(n3, allMediaTrayNames[j]);
                }
                else {
                    mediaTrays[n2++] = Win32PrintService.dmPaperBinToPrintService[n3 - 1];
                }
            }
            ++j;
        }
        this.mediaTrays = mediaTrays;
        this.gotTrays = true;
        return this.mediaTrays;
    }
    
    private boolean isSameSize(float n, float n2, float n3, float n4) {
        float a = n - n3;
        float a2 = n2 - n4;
        float a3 = n - n4;
        float a4 = n2 - n3;
        return (Math.abs(a) <= 1.0f && Math.abs(a2) <= 1.0f) || (Math.abs(a3) <= 1.0f && Math.abs(a4) <= 1.0f);
    }
    
    public MediaSizeName findMatchingMediaSizeNameMM(float n, float n2) {
        if (Win32PrintService.predefMedia != null) {
            for (int i = 0; i < Win32PrintService.predefMedia.length; ++i) {
                if (Win32PrintService.predefMedia[i] != null && this.isSameSize(Win32PrintService.predefMedia[i].getX(1000), Win32PrintService.predefMedia[i].getY(1000), n, n2)) {
                    return Win32PrintService.predefMedia[i].getMediaSizeName();
                }
            }
        }
        return null;
    }
    
    private MediaSize[] getMediaSizes(ArrayList list, int[] array, ArrayList<String> list2) {
        if (list2 == null) {
            list2 = new ArrayList();
        }
        String port = this.getPort();
        int[] allMediaSizes = this.getAllMediaSizes(this.printer, port);
        String[] allMediaNames = this.getAllMediaNames(this.printer, port);
        MediaSize mediaSizeForName = null;
        if (allMediaSizes == null || allMediaNames == null) {
            return null;
        }
        int n = allMediaSizes.length / 2;
        ArrayList o = new ArrayList();
        for (int i = 0; i < n; ++i, mediaSizeForName = null) {
            float x = (float)allMediaSizes[i * 2] / 10.0f;
            float y = (float)allMediaSizes[i * 2 + 1] / 10.0f;
            if (x <= 0.0f || y <= 0.0f) {
                if (n == array.length) {
                    list.remove(list.indexOf(Integer.valueOf(array[i])));
                }
            }
            else {
                MediaSizeName matchingMediaSizeNameMM = this.findMatchingMediaSizeNameMM(x, y);
                if (matchingMediaSizeNameMM != null) {
                    mediaSizeForName = MediaSize.getMediaSizeForName(matchingMediaSizeNameMM);
                }
                if (mediaSizeForName != null) {
                    o.add(mediaSizeForName);
                    list2.add(allMediaNames[i]);
                }
                else {
                    Win32MediaSize mediaName = Win32MediaSize.findMediaName(allMediaNames[i]);
                    if (mediaName == null) {
                        mediaName = new Win32MediaSize(allMediaNames[i], array[i]);
                    }
                    try {
                        o.add(new MediaSize(x, y, 1000, mediaName));
                        list2.add(allMediaNames[i]);
                    }
                    catch (IllegalArgumentException ex) {
                        if (n == array.length) {
                            list.remove(list.indexOf(Integer.valueOf(array[i])));
                        }
                    }
                }
            }
        }
        MediaSize[] a = new MediaSize[o.size()];
        o.toArray(a);
        return a;
    }
    
    private PrinterIsAcceptingJobs getPrinterIsAcceptingJobs() {
        if (this.getJobStatus(this.printer, 2) != 1) {
            return PrinterIsAcceptingJobs.NOT_ACCEPTING_JOBS;
        }
        return PrinterIsAcceptingJobs.ACCEPTING_JOBS;
    }
    
    private PrinterState getPrinterState() {
        if (this.isInvalid) {
            return PrinterState.STOPPED;
        }
        return null;
    }
    
    private PrinterStateReasons getPrinterStateReasons() {
        if (this.isInvalid) {
            PrinterStateReasons printerStateReasons = new PrinterStateReasons();
            printerStateReasons.put(PrinterStateReason.SHUTDOWN, Severity.ERROR);
            return printerStateReasons;
        }
        return null;
    }
    
    private QueuedJobCount getQueuedJobCount() {
        int jobStatus = this.getJobStatus(this.printer, 1);
        if (jobStatus != -1) {
            return new QueuedJobCount(jobStatus);
        }
        return new QueuedJobCount(0);
    }
    
    private boolean isSupportedCopies(Copies copies) {
        synchronized (this) {
            if (!this.gotCopies) {
                this.nCopies = this.getCopiesSupported(this.printer, this.getPort());
                this.gotCopies = true;
            }
        }
        int value = copies.getValue();
        return value > 0 && value <= this.nCopies;
    }
    
    private boolean isSupportedMedia(MediaSizeName mediaSizeName) {
        this.initMedia();
        if (this.mediaSizeNames != null) {
            int i = 0;
            while (i < this.mediaSizeNames.length) {
                if (mediaSizeName.equals(this.mediaSizeNames[i])) {
                    return true;
                }
                ++i;
            }
        }
        return false;
    }
    
    private boolean isSupportedMediaPrintableArea(MediaPrintableArea mediaPrintableArea) {
        this.getMediaPrintables(null);
        if (this.mediaPrintables != null) {
            int i = 0;
            while (i < this.mediaPrintables.length) {
                if (mediaPrintableArea.equals(this.mediaPrintables[i])) {
                    return true;
                }
                ++i;
            }
        }
        return false;
    }
    
    private boolean isSupportedMediaTray(MediaTray mediaTray) {
        MediaTray[] mediaTrays = this.getMediaTrays();
        if (mediaTrays != null) {
            int i = 0;
            while (i < mediaTrays.length) {
                if (mediaTray.equals(mediaTrays[i])) {
                    return true;
                }
                ++i;
            }
        }
        return false;
    }
    
    private int getPrinterCapabilities() {
        if (this.prnCaps == 0) {
            this.prnCaps = this.getCapabilities(this.printer, this.getPort());
        }
        return this.prnCaps;
    }
    
    private String getPort() {
        if (this.port == null) {
            this.port = this.getPrinterPort(this.printer);
        }
        return this.port;
    }
    
    private int[] getDefaultPrinterSettings() {
        if (this.defaultSettings == null) {
            this.defaultSettings = this.getDefaultSettings(this.printer, this.getPort());
        }
        return this.defaultSettings;
    }
    
    private PrinterResolution[] getPrintResolutions() {
        if (this.printRes == null) {
            int[] allResolutions = this.getAllResolutions(this.printer, this.getPort());
            if (allResolutions == null) {
                this.printRes = new PrinterResolution[0];
            }
            else {
                int n = allResolutions.length / 2;
                ArrayList<PrinterResolution> list = new ArrayList();
                for (int i = 0; i < n; ++i) {
                    try {
                        list.add(new PrinterResolution(allResolutions[i * 2], allResolutions[i * 2 + 1], 100));
                    }
                    catch (IllegalArgumentException ex) {}
                }
                this.printRes = (PrinterResolution[])(PrinterResolution[])list.toArray(new PrinterResolution[list.size()]);
            }
        }
        return this.printRes;
    }
    
    private boolean isSupportedResolution(PrinterResolution printerResolution) {
        PrinterResolution[] printResolutions = this.getPrintResolutions();
        if (printResolutions != null) {
            int i = 0;
            while (i < printResolutions.length) {
                if (printerResolution.equals(printResolutions[i])) {
                    return true;
                }
                ++i;
            }
        }
        return false;
    }
    
    @Override
    public DocPrintJob createPrintJob() {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPrintJobAccess();
        }
        return new Win32PrintJob(this);
    }
    
    private PrintServiceAttributeSet getDynamicAttributes() {
        HashPrintServiceAttributeSet set = new HashPrintServiceAttributeSet();
        set.add(this.getPrinterIsAcceptingJobs());
        set.add(this.getQueuedJobCount());
        return set;
    }
    
    @Override
    public PrintServiceAttributeSet getUpdatedAttributes() {
        PrintServiceAttributeSet dynamicAttributes = this.getDynamicAttributes();
        if (this.lastSet == null) {
            this.lastSet = dynamicAttributes;
            return AttributeSetUtilities.unmodifiableView(dynamicAttributes);
        }
        HashPrintServiceAttributeSet attributeSet = new HashPrintServiceAttributeSet();
        Attribute[] array = dynamicAttributes.toArray();
        for (int i = 0; i < array.length; ++i) {
            Attribute attribute = array[i];
            if (!this.lastSet.containsValue(attribute)) {
                attributeSet.add(attribute);
            }
        }
        this.lastSet = dynamicAttributes;
        return AttributeSetUtilities.unmodifiableView(attributeSet);
    }
    
    public void wakeNotifier() {
        synchronized (this) {
            if (this.notifier != null) {
                this.notifier.wake();
            }
        }
    }
    
    @Override
    public void addPrintServiceAttributeListener(PrintServiceAttributeListener printServiceAttributeListener) {
        synchronized (this) {
            if (printServiceAttributeListener == null) {
                return;
            }
            if (this.notifier == null) {
                this.notifier = new ServiceNotifier(this);
            }
            this.notifier.addListener(printServiceAttributeListener);
        }
    }
    
    @Override
    public void removePrintServiceAttributeListener(PrintServiceAttributeListener printServiceAttributeListener) {
        synchronized (this) {
            if (printServiceAttributeListener == null || this.notifier == null) {
                return;
            }
            this.notifier.removeListener(printServiceAttributeListener);
            if (this.notifier.isEmpty()) {
                this.notifier.stopNotifier();
                this.notifier = null;
            }
        }
    }
    
    @Override
    public <T extends PrintServiceAttribute> T getAttribute(Class<T> clazz) {
        if (clazz == null) {
            throw new NullPointerException("category");
        }
        if (!PrintServiceAttribute.class.isAssignableFrom(clazz)) {
            throw new IllegalArgumentException("Not a PrintServiceAttribute");
        }
        if (clazz == ColorSupported.class) {
            if ((this.getPrinterCapabilities() & 0x1) != 0x0) {
                return ColorSupported.SUPPORTED;
            }
            return ColorSupported.NOT_SUPPORTED;
        }
        else {
            if (clazz == PrinterName.class) {
                return this.getPrinterName();
            }
            if (clazz == PrinterState.class) {
                return this.getPrinterState();
            }
            if (clazz == PrinterStateReasons.class) {
                return this.getPrinterStateReasons();
            }
            if (clazz == QueuedJobCount.class) {
                return this.getQueuedJobCount();
            }
            if (clazz == PrinterIsAcceptingJobs.class) {
                return this.getPrinterIsAcceptingJobs();
            }
            return null;
        }
    }
    
    @Override
    public PrintServiceAttributeSet getAttributes() {
        HashPrintServiceAttributeSet attributeSet = new HashPrintServiceAttributeSet();
        attributeSet.add(this.getPrinterName());
        attributeSet.add(this.getPrinterIsAcceptingJobs());
        PrinterState printerState = this.getPrinterState();
        if (printerState != null) {
            attributeSet.add(printerState);
        }
        PrinterStateReasons printerStateReasons = this.getPrinterStateReasons();
        if (printerStateReasons != null) {
            attributeSet.add(printerStateReasons);
        }
        attributeSet.add(this.getQueuedJobCount());
        if ((this.getPrinterCapabilities() & 0x1) != 0x0) {
            attributeSet.add(ColorSupported.SUPPORTED);
        }
        else {
            attributeSet.add(ColorSupported.NOT_SUPPORTED);
        }
        return AttributeSetUtilities.unmodifiableView(attributeSet);
    }
    
    @Override
    public DocFlavor[] getSupportedDocFlavors() {
        int length = Win32PrintService.supportedFlavors.length;
        if ((this.getPrinterCapabilities() & 0x10) != 0x0) {
            DocFlavor[] array = new DocFlavor[length + 3];
            System.arraycopy(Win32PrintService.supportedFlavors, 0, array, 0, length);
            array[length] = DocFlavor.BYTE_ARRAY.POSTSCRIPT;
            array[length + 1] = DocFlavor.INPUT_STREAM.POSTSCRIPT;
            array[length + 2] = DocFlavor.URL.POSTSCRIPT;
            return array;
        }
        DocFlavor[] array = new DocFlavor[length];
        System.arraycopy(Win32PrintService.supportedFlavors, 0, array, 0, length);
        return array;
    }
    
    @Override
    public boolean isDocFlavorSupported(DocFlavor docFlavor) {
        DocFlavor[] array;
        if (this.isPostScriptFlavor(docFlavor)) {
            array = this.getSupportedDocFlavors();
        }
        else {
            array = Win32PrintService.supportedFlavors;
        }
        int i = 0;
        while (i < array.length) {
            if (docFlavor.equals(array[i])) {
                return true;
            }
            ++i;
        }
        return false;
    }
    
    @Override
    public Class<?>[] getSupportedAttributeCategories() {
        ArrayList<Object> list = new ArrayList(Win32PrintService.otherAttrCats.length + 3);
        for (int i = 0; i < Win32PrintService.otherAttrCats.length; ++i) {
            list.add(Win32PrintService.otherAttrCats[i]);
        }
        int printerCapabilities = this.getPrinterCapabilities();
        if ((printerCapabilities & 0x2) != 0x0) {
            list.add(Sides.class);
        }
        if ((printerCapabilities & 0x8) != 0x0) {
            int[] defaultPrinterSettings = this.getDefaultPrinterSettings();
            if (defaultPrinterSettings[3] >= -4 && defaultPrinterSettings[3] < 0) {
                list.add(PrintQuality.class);
            }
        }
        PrinterResolution[] printResolutions = this.getPrintResolutions();
        if (printResolutions != null && printResolutions.length > 0) {
            list.add(PrinterResolution.class);
        }
        return (Class[])(Class[])list.toArray(new Class[list.size()]);
    }
    
    @Override
    public boolean isAttributeCategorySupported(Class<? extends Attribute> obj) {
        if (obj == null) {
            throw new NullPointerException("null category");
        }
        if (!Attribute.class.isAssignableFrom(obj)) {
            throw new IllegalArgumentException(obj + " is not an Attribute");
        }
        Class[] supportedAttributeCategories = this.getSupportedAttributeCategories();
        int i = 0;
        while (i < supportedAttributeCategories.length) {
            if (obj.equals(supportedAttributeCategories[i])) {
                return true;
            }
            ++i;
        }
        return false;
    }
    
    @Override
    public Object getDefaultAttributeValue(Class<? extends Attribute> obj) {
        if (obj == null) {
            throw new NullPointerException("null category");
        }
        if (!Attribute.class.isAssignableFrom(obj)) {
            throw new IllegalArgumentException(obj + " is not an Attribute");
        }
        if (!this.isAttributeCategorySupported(obj)) {
            return null;
        }
        int[] defaultPrinterSettings = this.getDefaultPrinterSettings();
        int paperID = defaultPrinterSettings[0];
        int feedResolution = defaultPrinterSettings[2];
        int crossFeedResolution = defaultPrinterSettings[3];
        int value = defaultPrinterSettings[4];
        int n = defaultPrinterSettings[5];
        int n2 = defaultPrinterSettings[6];
        int n3 = defaultPrinterSettings[7];
        int n4 = defaultPrinterSettings[8];
        if (obj == Copies.class) {
            if (value > 0) {
                return new Copies(value);
            }
            return new Copies(1);
        }
        else if (obj == Chromaticity.class) {
            if (n4 == 2) {
                return Chromaticity.COLOR;
            }
            return Chromaticity.MONOCHROME;
        }
        else {
            if (obj == JobName.class) {
                return new JobName("Java Printing", null);
            }
            if (obj == OrientationRequested.class) {
                if (n == 2) {
                    return OrientationRequested.LANDSCAPE;
                }
                return OrientationRequested.PORTRAIT;
            }
            else {
                if (obj == PageRanges.class) {
                    return new PageRanges(1, Integer.MAX_VALUE);
                }
                if (obj == Media.class) {
                    MediaSizeName win32Media2 = this.findWin32Media(paperID);
                    if (win32Media2 != null) {
                        if (!this.isSupportedMedia(win32Media2) && this.mediaSizeNames != null) {
                            win32Media2 = this.mediaSizeNames[0];
                            this.findPaperID(win32Media2);
                            return win32Media2;
                        }
                        return win32Media2;
                    }
                    else {
                        this.initMedia();
                        if (this.mediaSizeNames != null && this.mediaSizeNames.length > 0) {
                            if (this.idList != null && this.mediaSizes != null && this.idList.size() == this.mediaSizes.length) {
                                int index = this.idList.indexOf(Integer.valueOf(paperID));
                                if (index >= 0 && index < this.mediaSizes.length) {
                                    return this.mediaSizes[index].getMediaSizeName();
                                }
                            }
                            return this.mediaSizeNames[0];
                        }
                    }
                }
                else if (obj == MediaPrintableArea.class) {
                    MediaSizeName win32Media = this.findWin32Media(paperID);
                    if (win32Media != null && !this.isSupportedMedia(win32Media) && this.mediaSizeNames != null) {
                        paperID = this.findPaperID(this.mediaSizeNames[0]);
                    }
                    float[] mediaPrintableArea = this.getMediaPrintableArea(this.printer, paperID);
                    if (mediaPrintableArea != null) {
                        Object o = null;
                        try {
                            o = new MediaPrintableArea(mediaPrintableArea[0], mediaPrintableArea[1], mediaPrintableArea[2], mediaPrintableArea[3], 25400);
                        }
                        catch (IllegalArgumentException ex4) {}
                        return o;
                    }
                    return null;
                }
                else {
                    if (obj == SunAlternateMedia.class) {
                        return null;
                    }
                    if (obj == Destination.class) {
                        try {
                            return new Destination(new File("out.prn").toURI());
                        }
                        catch (SecurityException ex) {
                            try {
                                return new Destination(new URI("file:out.prn"));
                            }
                            catch (URISyntaxException ex2) {
                                return null;
                            }
                        }
                    }
                    if (obj == Sides.class) {
                        switch (n2) {
                            case 2: {
                                return Sides.TWO_SIDED_LONG_EDGE;
                            }
                            case 3: {
                                return Sides.TWO_SIDED_SHORT_EDGE;
                            }
                            default: {
                                return Sides.ONE_SIDED;
                            }
                        }
                    }
                    else if (obj == PrinterResolution.class) {
                        if (crossFeedResolution < 0 || feedResolution < 0) {
                            int n5 = (feedResolution > crossFeedResolution) ? feedResolution : crossFeedResolution;
                            if (n5 > 0) {
                                return new PrinterResolution(n5, n5, 100);
                            }
                        }
                        else {
                            return new PrinterResolution(crossFeedResolution, feedResolution, 100);
                        }
                    }
                    else if (obj == ColorSupported.class) {
                        if ((this.getPrinterCapabilities() & 0x1) != 0x0) {
                            return ColorSupported.SUPPORTED;
                        }
                        return ColorSupported.NOT_SUPPORTED;
                    }
                    else if (obj == PrintQuality.class) {
                        if (crossFeedResolution < 0 && crossFeedResolution >= -4) {
                            switch (crossFeedResolution) {
                                case -4: {
                                    return PrintQuality.HIGH;
                                }
                                case -3: {
                                    return PrintQuality.NORMAL;
                                }
                                default: {
                                    return PrintQuality.DRAFT;
                                }
                            }
                        }
                    }
                    else {
                        if (obj == RequestingUserName.class) {
                            String property = "";
                            try {
                                property = System.getProperty("user.name", "");
                            }
                            catch (SecurityException ex3) {}
                            return new RequestingUserName(property, null);
                        }
                        if (obj == SheetCollate.class) {
                            if (n3 == 1) {
                                return SheetCollate.COLLATED;
                            }
                            return SheetCollate.UNCOLLATED;
                        }
                        else if (obj == Fidelity.class) {
                            return Fidelity.FIDELITY_FALSE;
                        }
                    }
                }
                return null;
            }
        }
    }
    
    private boolean isPostScriptFlavor(DocFlavor docFlavor) {
        return docFlavor.equals(DocFlavor.BYTE_ARRAY.POSTSCRIPT) || docFlavor.equals(DocFlavor.INPUT_STREAM.POSTSCRIPT) || docFlavor.equals(DocFlavor.URL.POSTSCRIPT);
    }
    
    private boolean isPSDocAttr(Class clazz) {
        return clazz == OrientationRequested.class || clazz == Copies.class;
    }
    
    private boolean isAutoSense(DocFlavor docFlavor) {
        return docFlavor.equals(DocFlavor.BYTE_ARRAY.AUTOSENSE) || docFlavor.equals(DocFlavor.INPUT_STREAM.AUTOSENSE) || docFlavor.equals(DocFlavor.URL.AUTOSENSE);
    }
    
    @Override
    public Object getSupportedAttributeValues(Class<? extends Attribute> obj, DocFlavor obj2, AttributeSet set) {
        if (obj == null) {
            throw new NullPointerException("null category");
        }
        if (!Attribute.class.isAssignableFrom(obj)) {
            throw new IllegalArgumentException(obj + " does not implement Attribute");
        }
        if (obj2 != null) {
            if (!this.isDocFlavorSupported(obj2)) {
                throw new IllegalArgumentException(obj2 + " is an unsupported flavor");
            }
            if (this.isAutoSense(obj2) || (this.isPostScriptFlavor(obj2) && this.isPSDocAttr(obj))) {
                return null;
            }
        }
        if (!this.isAttributeCategorySupported(obj)) {
            return null;
        }
        if (obj == JobName.class) {
            return new JobName("Java Printing", null);
        }
        if (obj == RequestingUserName.class) {
            String property = "";
            try {
                property = System.getProperty("user.name", "");
            }
            catch (SecurityException ex3) {}
            return new RequestingUserName(property, null);
        }
        if (obj == ColorSupported.class) {
            if ((this.getPrinterCapabilities() & 0x1) != 0x0) {
                return ColorSupported.SUPPORTED;
            }
            return ColorSupported.NOT_SUPPORTED;
        }
        else if (obj == Chromaticity.class) {
            if (obj2 == null || obj2.equals(DocFlavor.SERVICE_FORMATTED.PAGEABLE) || obj2.equals(DocFlavor.SERVICE_FORMATTED.PRINTABLE) || obj2.equals(DocFlavor.BYTE_ARRAY.GIF) || obj2.equals(DocFlavor.INPUT_STREAM.GIF) || obj2.equals(DocFlavor.URL.GIF) || obj2.equals(DocFlavor.BYTE_ARRAY.JPEG) || obj2.equals(DocFlavor.INPUT_STREAM.JPEG) || obj2.equals(DocFlavor.URL.JPEG) || obj2.equals(DocFlavor.BYTE_ARRAY.PNG) || obj2.equals(DocFlavor.INPUT_STREAM.PNG) || obj2.equals(DocFlavor.URL.PNG)) {
                if ((this.getPrinterCapabilities() & 0x1) == 0x0) {
                    return new Chromaticity[] { Chromaticity.MONOCHROME };
                }
                return new Chromaticity[] { Chromaticity.MONOCHROME, Chromaticity.COLOR };
            }
            else {
                return null;
            }
        }
        else {
            if (obj == Destination.class) {
                try {
                    return new Destination(new File("out.prn").toURI());
                }
                catch (SecurityException ex) {
                    try {
                        return new Destination(new URI("file:out.prn"));
                    }
                    catch (URISyntaxException ex2) {
                        return null;
                    }
                }
            }
            if (obj == OrientationRequested.class) {
                if (obj2 == null || obj2.equals(DocFlavor.SERVICE_FORMATTED.PAGEABLE) || obj2.equals(DocFlavor.SERVICE_FORMATTED.PRINTABLE) || obj2.equals(DocFlavor.INPUT_STREAM.GIF) || obj2.equals(DocFlavor.INPUT_STREAM.JPEG) || obj2.equals(DocFlavor.INPUT_STREAM.PNG) || obj2.equals(DocFlavor.BYTE_ARRAY.GIF) || obj2.equals(DocFlavor.BYTE_ARRAY.JPEG) || obj2.equals(DocFlavor.BYTE_ARRAY.PNG) || obj2.equals(DocFlavor.URL.GIF) || obj2.equals(DocFlavor.URL.JPEG) || obj2.equals(DocFlavor.URL.PNG)) {
                    return new OrientationRequested[] { OrientationRequested.PORTRAIT, OrientationRequested.LANDSCAPE, OrientationRequested.REVERSE_LANDSCAPE };
                }
                return null;
            }
            else {
                if (obj == Copies.class || obj == CopiesSupported.class) {
                    synchronized (this) {
                        if (!this.gotCopies) {
                            this.nCopies = this.getCopiesSupported(this.printer, this.getPort());
                            this.gotCopies = true;
                        }
                    }
                    return new CopiesSupported(1, this.nCopies);
                }
                if (obj == Media.class) {
                    this.initMedia();
                    int n = (this.mediaSizeNames == null) ? 0 : this.mediaSizeNames.length;
                    MediaTray[] mediaTrays = this.getMediaTrays();
                    int n2 = n + ((mediaTrays == null) ? 0 : mediaTrays.length);
                    Media[] array3 = new Media[n2];
                    if (this.mediaSizeNames != null) {
                        System.arraycopy(this.mediaSizeNames, 0, array3, 0, this.mediaSizeNames.length);
                    }
                    if (mediaTrays != null) {
                        System.arraycopy(mediaTrays, 0, array3, n2 - mediaTrays.length, mediaTrays.length);
                        return array3;
                    }
                    return array3;
                }
                else if (obj == MediaPrintableArea.class) {
                    Media media = null;
                    if (set != null && (media = (Media)set.get(Media.class)) != null && !(media instanceof MediaSizeName)) {
                        media = null;
                    }
                    MediaPrintableArea[] mediaPrintables = this.getMediaPrintables((MediaSizeName)media);
                    if (mediaPrintables != null) {
                        MediaPrintableArea[] array2 = new MediaPrintableArea[mediaPrintables.length];
                        System.arraycopy(mediaPrintables, 0, array2, 0, mediaPrintables.length);
                        return array2;
                    }
                    return null;
                }
                else {
                    if (obj == SunAlternateMedia.class) {
                        return new SunAlternateMedia((Media)this.getDefaultAttributeValue(Media.class));
                    }
                    if (obj == PageRanges.class) {
                        if (obj2 == null || obj2.equals(DocFlavor.SERVICE_FORMATTED.PAGEABLE) || obj2.equals(DocFlavor.SERVICE_FORMATTED.PRINTABLE)) {
                            return new PageRanges[] { new PageRanges(1, Integer.MAX_VALUE) };
                        }
                        return null;
                    }
                    else if (obj == PrinterResolution.class) {
                        PrinterResolution[] printResolutions = this.getPrintResolutions();
                        if (printResolutions == null) {
                            return null;
                        }
                        PrinterResolution[] array = new PrinterResolution[printResolutions.length];
                        System.arraycopy(printResolutions, 0, array, 0, printResolutions.length);
                        return array;
                    }
                    else if (obj == Sides.class) {
                        if (obj2 == null || obj2.equals(DocFlavor.SERVICE_FORMATTED.PAGEABLE) || obj2.equals(DocFlavor.SERVICE_FORMATTED.PRINTABLE)) {
                            return new Sides[] { Sides.ONE_SIDED, Sides.TWO_SIDED_LONG_EDGE, Sides.TWO_SIDED_SHORT_EDGE };
                        }
                        return null;
                    }
                    else {
                        if (obj == PrintQuality.class) {
                            return new PrintQuality[] { PrintQuality.DRAFT, PrintQuality.HIGH, PrintQuality.NORMAL };
                        }
                        if (obj == SheetCollate.class) {
                            if (obj2 == null || obj2.equals(DocFlavor.SERVICE_FORMATTED.PAGEABLE) || obj2.equals(DocFlavor.SERVICE_FORMATTED.PRINTABLE)) {
                                return new SheetCollate[] { SheetCollate.COLLATED, SheetCollate.UNCOLLATED };
                            }
                            return null;
                        }
                        else {
                            if (obj == Fidelity.class) {
                                return new Fidelity[] { Fidelity.FIDELITY_FALSE, Fidelity.FIDELITY_TRUE };
                            }
                            return null;
                        }
                    }
                }
            }
        }
    }
    
    @Override
    public boolean isAttributeValueSupported(Attribute attribute, DocFlavor obj, AttributeSet set) {
        if (attribute == null) {
            throw new NullPointerException("null attribute");
        }
        Class<Object> category = attribute.getCategory();
        if (obj != null) {
            if (!this.isDocFlavorSupported(obj)) {
                throw new IllegalArgumentException(obj + " is an unsupported flavor");
            }
            if (this.isAutoSense(obj) || (this.isPostScriptFlavor(obj) && this.isPSDocAttr(category))) {
                return false;
            }
        }
        if (!this.isAttributeCategorySupported(category)) {
            return false;
        }
        if (category == Chromaticity.class) {
            return (obj == null || obj.equals(DocFlavor.SERVICE_FORMATTED.PAGEABLE) || obj.equals(DocFlavor.SERVICE_FORMATTED.PRINTABLE) || obj.equals(DocFlavor.BYTE_ARRAY.GIF) || obj.equals(DocFlavor.INPUT_STREAM.GIF) || obj.equals(DocFlavor.URL.GIF) || obj.equals(DocFlavor.BYTE_ARRAY.JPEG) || obj.equals(DocFlavor.INPUT_STREAM.JPEG) || obj.equals(DocFlavor.URL.JPEG) || obj.equals(DocFlavor.BYTE_ARRAY.PNG) || obj.equals(DocFlavor.INPUT_STREAM.PNG) || obj.equals(DocFlavor.URL.PNG)) && ((this.getPrinterCapabilities() & 0x1) != 0x0 || attribute == Chromaticity.MONOCHROME);
        }
        if (category == Copies.class) {
            return this.isSupportedCopies((Copies)attribute);
        }
        if (category == Destination.class) {
            URI uri = ((Destination)attribute).getURI();
            return "file".equals(uri.getScheme()) && !uri.getSchemeSpecificPart().equals("");
        }
        if (category == Media.class) {
            if ((attribute instanceof MediaSizeName)) {
                return this.isSupportedMedia((MediaSizeName)attribute);
            }
            if ((attribute instanceof MediaTray)) {
                return this.isSupportedMediaTray((MediaTray)attribute);
            }
        }
        else {
            if (category == MediaPrintableArea.class) {
                return this.isSupportedMediaPrintableArea((MediaPrintableArea)attribute);
            }
            if (category == SunAlternateMedia.class) {
                return this.isAttributeValueSupported(((SunAlternateMedia)attribute).getMedia(), obj, set);
            }
            if (category == PageRanges.class || category == SheetCollate.class || category == Sides.class) {
                if (obj != null && !obj.equals(DocFlavor.SERVICE_FORMATTED.PAGEABLE) && !obj.equals(DocFlavor.SERVICE_FORMATTED.PRINTABLE)) {
                    return false;
                }
            }
            else if (category == PrinterResolution.class) {
                if ((attribute instanceof PrinterResolution)) {
                    return this.isSupportedResolution((PrinterResolution)attribute);
                }
            }
            else if (category == OrientationRequested.class) {
                if (attribute == OrientationRequested.REVERSE_PORTRAIT || (obj != null && !obj.equals(DocFlavor.SERVICE_FORMATTED.PAGEABLE) && !obj.equals(DocFlavor.SERVICE_FORMATTED.PRINTABLE) && !obj.equals(DocFlavor.INPUT_STREAM.GIF) && !obj.equals(DocFlavor.INPUT_STREAM.JPEG) && !obj.equals(DocFlavor.INPUT_STREAM.PNG) && !obj.equals(DocFlavor.BYTE_ARRAY.GIF) && !obj.equals(DocFlavor.BYTE_ARRAY.JPEG) && !obj.equals(DocFlavor.BYTE_ARRAY.PNG) && !obj.equals(DocFlavor.URL.GIF) && !obj.equals(DocFlavor.URL.JPEG) && !obj.equals(DocFlavor.URL.PNG))) {
                    return false;
                }
            }
            else if (category == ColorSupported.class) {
                int n = (this.getPrinterCapabilities() & 0x1) != 0x0;
                if ((n == 0 && attribute == ColorSupported.SUPPORTED) || (n && attribute == ColorSupported.NOT_SUPPORTED)) {
                    return false;
                }
            }
        }
        return true;
    }
    
    @Override
    public AttributeSet getUnsupportedAttributes(DocFlavor obj, AttributeSet set) {
        if (obj != null && !this.isDocFlavorSupported(obj)) {
            throw new IllegalArgumentException("flavor " + obj + "is not supported");
        }
        if (set == null) {
            return null;
        }
        HashAttributeSet set2 = new HashAttributeSet();
        Attribute[] array = set.toArray();
        for (int i = 0; i < array.length; ++i) {
            try {
                Attribute attribute = array[i];
                if (!this.isAttributeCategorySupported(attribute.getCategory())) {
                    set2.add(attribute);
                }
                else if (!this.isAttributeValueSupported(attribute, obj, set)) {
                    set2.add(attribute);
                }
            }
            catch (ClassCastException ex) {}
        }
        if (set2.isEmpty()) {
            return null;
        }
        return set2;
    }
    
    private synchronized DocumentPropertiesUI getDocumentPropertiesUI() {
        return new Win32PrintService.Win32DocumentPropertiesUI(this);
    }
    
    @Override
    public synchronized ServiceUIFactory getServiceUIFactory() {
        if (this.uiFactory == null) {
            this.uiFactory = new Win32PrintService.Win32ServiceUIFactory(this);
        }
        return this.uiFactory;
    }
    
    @Override
    public String toString() {
        return "Win32 Printer : " + this.getName();
    }
    
    @Override
    public boolean equals(Object o) {
        return o == this || (o instanceof Win32PrintService && ((Win32PrintService)o).getName().equals(this.getName()));
    }
    
    @Override
    public int hashCode() {
        return this.getClass().hashCode() + this.getName().hashCode();
    }
    
    @Override
    public boolean usesClass(Class clazz) {
        return clazz == WPrinterJob.class;
    }
    
    private native int[] getAllMediaIDs(String p0, String p1);
    
    private native int[] getAllMediaSizes(String p0, String p1);
    
    private native int[] getAllMediaTrays(String p0, String p1);
    
    private native float[] getMediaPrintableArea(String p0, int p1);
    
    private native String[] getAllMediaNames(String p0, String p1);
    
    private native String[] getAllMediaTrayNames(String p0, String p1);
    
    private native int getCopiesSupported(String p0, String p1);
    
    private native int[] getAllResolutions(String p0, String p1);
    
    private native int getCapabilities(String p0, String p1);
    
    private native int[] getDefaultSettings(String p0, String p1);
    
    private native int getJobStatus(String p0, int p1);
    
    private native String getPrinterPort(String p0);
    
    static DocumentPropertiesUI access$100(Win32PrintService win32PrintService) {
        return win32PrintService.getDocumentPropertiesUI();
    }
    
    static {
        Win32PrintService.predefMedia = Win32MediaSize.getPredefMedia();
        supportedFlavors = new DocFlavor[] { DocFlavor.BYTE_ARRAY.GIF, DocFlavor.INPUT_STREAM.GIF, DocFlavor.URL.GIF, DocFlavor.BYTE_ARRAY.JPEG, DocFlavor.INPUT_STREAM.JPEG, DocFlavor.URL.JPEG, DocFlavor.BYTE_ARRAY.PNG, DocFlavor.INPUT_STREAM.PNG, DocFlavor.URL.PNG, DocFlavor.SERVICE_FORMATTED.PAGEABLE, DocFlavor.SERVICE_FORMATTED.PRINTABLE, DocFlavor.BYTE_ARRAY.AUTOSENSE, DocFlavor.URL.AUTOSENSE, DocFlavor.INPUT_STREAM.AUTOSENSE };
        serviceAttrCats = new Class[] { PrinterName.class, PrinterIsAcceptingJobs.class, QueuedJobCount.class, ColorSupported.class };
        Win32PrintService.otherAttrCats = new Class[] { JobName.class, RequestingUserName.class, Copies.class, Destination.class, OrientationRequested.class, PageRanges.class, Media.class, MediaPrintableArea.class, Fidelity.class, SheetCollate.class, SunAlternateMedia.class, Chromaticity.class };
        dmPaperToPrintService = new MediaSizeName[] { MediaSizeName.NA_LETTER, MediaSizeName.NA_LETTER, MediaSizeName.TABLOID, MediaSizeName.LEDGER, MediaSizeName.NA_LEGAL, MediaSizeName.INVOICE, MediaSizeName.EXECUTIVE, MediaSizeName.ISO_A3, MediaSizeName.ISO_A4, MediaSizeName.ISO_A4, MediaSizeName.ISO_A5, MediaSizeName.JIS_B4, MediaSizeName.JIS_B5, MediaSizeName.FOLIO, MediaSizeName.QUARTO, MediaSizeName.NA_10X14_ENVELOPE, MediaSizeName.B, MediaSizeName.NA_LETTER, MediaSizeName.NA_NUMBER_9_ENVELOPE, MediaSizeName.NA_NUMBER_10_ENVELOPE, MediaSizeName.NA_NUMBER_11_ENVELOPE, MediaSizeName.NA_NUMBER_12_ENVELOPE, MediaSizeName.NA_NUMBER_14_ENVELOPE, MediaSizeName.C, MediaSizeName.D, MediaSizeName.E, MediaSizeName.ISO_DESIGNATED_LONG, MediaSizeName.ISO_C5, MediaSizeName.ISO_C3, MediaSizeName.ISO_C4, MediaSizeName.ISO_C6, MediaSizeName.ITALY_ENVELOPE, MediaSizeName.ISO_B4, MediaSizeName.ISO_B5, MediaSizeName.ISO_B6, MediaSizeName.ITALY_ENVELOPE, MediaSizeName.MONARCH_ENVELOPE, MediaSizeName.PERSONAL_ENVELOPE, MediaSizeName.NA_10X15_ENVELOPE, MediaSizeName.NA_9X12_ENVELOPE, MediaSizeName.FOLIO, MediaSizeName.ISO_B4, MediaSizeName.JAPANESE_POSTCARD, MediaSizeName.NA_9X11_ENVELOPE };
        dmPaperBinToPrintService = new MediaTray[] { MediaTray.TOP, MediaTray.BOTTOM, MediaTray.MIDDLE, MediaTray.MANUAL, MediaTray.ENVELOPE, Win32MediaTray.ENVELOPE_MANUAL, Win32MediaTray.AUTO, Win32MediaTray.TRACTOR, Win32MediaTray.SMALL_FORMAT, Win32MediaTray.LARGE_FORMAT, MediaTray.LARGE_CAPACITY, null, null, MediaTray.MAIN, Win32MediaTray.FORMSOURCE };
        Win32PrintService.DM_PAPERSIZE = 2;
        Win32PrintService.DM_PRINTQUALITY = 1024;
        Win32PrintService.DM_YRESOLUTION = 8192;
    }
    
    private static class Win32DocumentPropertiesUI extends DocumentPropertiesUI
    {
        Win32PrintService service;
        
        private Win32DocumentPropertiesUI(Win32PrintService service) {
            this.service = service;
        }
        
        @Override
        public PrintRequestAttributeSet showDocumentProperties(PrinterJob printerJob, Window window, PrintService printService, PrintRequestAttributeSet set) {
            if (!(printerJob instanceof WPrinterJob)) {
                return null;
            }
            return ((WPrinterJob)printerJob).showDocumentProperties(window, printService, set);
        }
    }
    
    private static class Win32ServiceUIFactory extends ServiceUIFactory
    {
        Win32PrintService service;
        
        Win32ServiceUIFactory(Win32PrintService service) {
            this.service = service;
        }
        
        @Override
        public Object getUI(int n, String anObject) {
            if (n <= 3) {
                return null;
            }
            if (n == 199 && DocumentPropertiesUI.DOCPROPERTIESCLASSNAME.equals(anObject)) {
                return Win32PrintService.access$100(this.service);
            }
            throw new IllegalArgumentException("Unsupported role");
        }
        
        @Override
        public String[] getUIClassNamesForRole(int n) {
            if (n <= 3) {
                return null;
            }
            if (n == 199) {
                String[] array = new String[0];
                array[0] = DocumentPropertiesUI.DOCPROPERTIESCLASSNAME;
                return array;
            }
            throw new IllegalArgumentException("Unsupported role");
        }
    }
}
